from botshop.pytorch.conversation_engine import BasicConversationEngine
from botshop.sample_and_rank import SampleAndRank


class SampleAndRankConversationEngine(SampleAndRank, BasicConversationEngine):
    pass
